library lifegame;

uses Windows, NkMonAPI;

{$E nmp}

type
  PIntArray = ^TIntArray;
  TIntArray = array[0..0] of Integer;
  PInteger = ^Integer;
  TLifeGame = class(TObject)
    X: Integer;
    Y: Integer;
    Width: Integer;
    Height: Integer;
    Grid0: PIntArray;
    Grid1: PIntArray;
    Cells: Integer;
    PrevCells: Integer;
    World: Integer;
    Gen: Integer;
    MaxGen: Integer;
    Stable: Integer;
    CellPercent: Integer;
    Color: Integer;
    MaxCells: Integer;
    Bits, Mask: Pointer;
    constructor Create(AWidth, AHeight: Integer);
    destructor Destroy; override;
    procedure GenWorld(ACellPercent: Integer);
    procedure Update;
    procedure UpdateBits;
  end;

constructor TLifeGame.Create(AWidth, AHeight: Integer);
begin
  Randomize;
  Width := AWidth;
  Height := AHeight;
  GetMem(Grid0, Width * Height * SizeOf(Integer));
  GetMem(Grid1, Width * Height * SizeOf(Integer));
  GetMem(Bits, Width * Height * SizeOf(Integer));
  GetMem(Mask, Width * Height * SizeOf(Integer));
end;

destructor TLifeGame.Destroy;
begin
  FreeMem(Grid0);
  FreeMem(Grid1);
  FreeMem(Bits);
  FreeMem(Mask);
end;

procedure TLifeGame.GenWorld(ACellPercent: Integer);
var
  i: Integer;
  p: PInteger;
begin
  Gen := 0;
  Stable := 0;
  CellPercent := ACellPercent;
  p := @Grid0[0];
  for i := 0 to Width*Height-1 do
  begin
    p^ := Ord(Random(101) <= CellPercent);
    Inc(p);
  end;
  Inc(World);
end;

procedure TLifeGame.Update;
var
  cx, cy, lx, rx, n: Integer;
  cline, uline, dline, tmp: PIntArray;
  g0, g1: PInteger;
begin
  if ((PrevCells - Cells) >= -4) and ((PrevCells - Cells) <= 4) then
    Inc(Stable)
  else
    Stable := 0;
  if Stable = 10 then
    GenWorld(CellPercent);
  PrevCells := Cells;

  g0 := @Grid0[0];
  g1 := @Grid1[0];
  Cells := 0;
  for cy := 0 to Height-1 do
  begin
    if cy = 0 then  
      uline := @Grid0[(Height-1)*Width]
    else
      uline := @Grid0[(cy-1)*Width];
    cline := @Grid0[cy*Width];
    if cy = (Height-1) then  
      dline := Grid0
    else
      dline := @Grid0[(cy+1)*Width];
    for cx := 0 to Width-1 do
    begin
      if cx = 0 then
        lx := Width-1
      else
        lx := cx-1;
      if cx = Width-1 then
        rx := 0
      else
        rx := cx+1;
      n := uline[lx] + uline[cx] + uline[rx] +
           cline[lx] +             cline[rx] +
           dline[lx] + dline[cx] + dline[rx];
      if n = 2 then
        g1^ := g0^
      else if n = 3 then
        g1^ := g0^ or 1
      else
        g1^ := 0;
      if g1^ = 1 then
        Inc(Cells);
      Inc(g0);
      Inc(g1);
    end;
  end;
  if MaxCells < Cells then
    MaxCells := Cells;

  Inc(Gen);
  if MaxGen < Gen then
    MaxGen := Gen;

  tmp := Grid0;
  Grid0 := Grid1;
  Grid1 := tmp;
  UpdateBits;
end;

procedure TLifeGame.UpdateBits;
var
  gr, bi, ma: PInteger;
  yy, xx: Integer;
begin
  gr := @Grid0[0];
  bi := Bits;
  ma := Mask;
  for yy := 0 to Height-1 do
    for xx := 0  to Width-1 do
    begin
       if gr^ > 0 then
       begin
         bi^ := Color;
         ma^ := 0;
       end
       else begin
         bi^ := 0;
         ma^ := $FFFFFF;
       end;
       Inc(gr);
       Inc(bi);
       Inc(ma);
    end;
end;

function init(X, Y, Width, Height, R, G, B: Integer; Argc: Integer;
  Args: Pointer): Integer; stdcall;
var
  lg: TLifeGame;
begin
  lg := TLifeGame.Create(Width, Height);
  lg.Color := ((R and $FF) shl 16) or ((G and $FF) shl 8) or (B and $FF);
  lg.GenWorld(10);
  Result := Integer(lg);
end;

procedure draw(Id: Integer; DC: HDC; X, Y, Width, Height, R, G, B: Integer); stdcall;
var
  lg: TLifeGame;
  bmi: TBitmapInfo;
begin
  lg := TLifeGame(Id);
  FillChar(bmi, SizeOf(bmi), 0);
  with bmi.bmiHeader do
  begin
    biSize := SizeOf(bmi.bmiHeader);
    biWidth := Width;
    biHeight := Height;
    biPlanes := 1;
    biBitCount := 32;
    biCompression := BI_RGB;
  end;
  StretchDIBits(DC, X, Y, Width, Height, 0, 0, lg.Width, lg.Height,
    lg.Mask, bmi, DIB_RGB_COLORS, SRCAND);
  StretchDIBits(DC, X, Y, Width, Height, 0, 0, lg.Width, lg.Height,
    lg.Bits, bmi, DIB_RGB_COLORS, SRCPAINT);
end;

procedure update(Id: Integer); stdcall;
begin
  with TLifeGame(Id) do
  begin
    Update;
    nkmSetValueInt('lifegame.world', World);
    nkmSetValueInt('lifegame.gen', Gen);
    nkmSetValueInt('lifegame.maxgen', MaxGen);
    nkmSetValueInt('lifegame.cells', Cells);
    nkmSetValueInt('lifegame.maxcells', MaxCells);
  end;
end;

procedure unload(Id: Integer); stdcall;
begin
  TLifeGame(Id).Free;
end;

exports
  init, draw, update, unload;

end.

 
