unit Smart;

interface

uses Windows;

const
  MAX_IDE_DRIVES = 4;
  READ_ATTRIBUTE_BUFFER_SIZE = 512;
  IDENTIFY_BUFFER_SIZE = 512;
  READ_THRESHOLD_BUFFER_SIZE = 512;

  // IOCTL commands
  DFP_GET_VERSION        = $00074080;
  DFP_SEND_DRIVE_COMMAND = $0007c084;
  DFP_RECEIVE_DRIVE_DATA = $0007c088;

type
  TGetVersionOutParams = packed record
    bVersion: Byte;
    bRevision: Byte;
    bReserved: Byte;
    bIDEDeviceMap: Byte;
    fCapabilities: DWORD;
    dwReserved: array[0..3] of DWORD;
  end;

const
  // Bits returned in the fCapabilities member of TGetVersionOutParams
  CAP_IDE_ID_FUNCTION = 1;	      // ATA ID command supported
  CAP_IDE_ATAPI_ID = 2;	              // ATAPI ID command supported
  CAP_IDE_EXECUTE_SMART_FUNCTION = 4; // SMART commannds supported

type
  // IDE registers
  TIdeRegs = packed record
    bFeaturesReg: Byte;
    bSectorCountReg: Byte;
    bSectorNumberReg: Byte;
    bCylLowReg: Byte;
    bCylHighReg: Byte;
    bDriveHeadReg: Byte;
    bCommandReg: Byte;
    bReserverd: Byte;
  end;

  TSendCmdInParams = packed record
    cBufferSize: DWORD;
    irDriveRegs: TIdeRegs;
    bDriveNumber: Byte; // 0..3
    bReserved: array[0..2] of Byte;
    dwReserved: array[0..4] of DWORD;
    bBuffer: array[0..0] of Byte;
  end;

const
  IDE_ATAPI_ID = $A1;               // Returns ID sector for ATAPI.
  IDE_ID_FUNCTION= $EC;             // Returns ID sector for ATA.
  IDE_EXECUTE_SMART_FUNCTION = $B0; // Performs SMART cmd.
                                    // Requires valid bFeaturesReg,
                                    // bCylLowReg, and bCylHighReg

  // Cylinder register values required when issuing SMART command
  SMART_CYL_LOW = $4F;
  SMART_CYL_HI = $C2;

type
  // Status returned from driver
  TDriverStatus = packed record
    bDriverError: Byte;
    bIDEStatus: Byte;
    bReserved: array[0..1] of Byte;
    dwReserved: array[0..1] of DWORD;
  end;

const
  // bDriverError values
  SMART_NO_ERROR         = 0;  // No error
  SMART_IDE_ERROR        = 1;  // Error from IDE controller
  SMART_INVALID_FLAG     = 2;  // Invalid command flag
  SMART_INVALID_COMMAND  = 3;  // Invalid command byte
  SMART_INVALID_BUFFER   = 4;  // Bad buffer (null, invalid addr..)
  SMART_INVALID_DRIVE    = 5;  // Drive number not valid
  SMART_INVALID_IOCTL    = 6;  // Invalid IOCTL
  SMART_ERROR_NO_MEM     = 7;  // Could not lock user's buffer
  SMART_INVALID_REGISTER = 8;  // Some IDE Register not valid
  SMART_NOT_SUPPORTED    = 9;  // Invalid cmd flag set
  SMART_NO_IDE_DEVICE    = 10; // Cmd issued to device not present
                               // although drive number is valid
  // 11-255 reserved

type
  PSendCmdOutParams = ^TSendCmdOutParams;
  TSendCmdOutParams = packed record
    cBufferSize: DWORD;
    DriverStatus: TDriverStatus;
    bBuffer: array[0..0] of Byte;
  end;

const
  // Feature register defines for SMART "sub commands"
  SMART_READ_ATTRIBUTE_VALUES             = $D0; // ATA4: Renamed SMART READ DATA
  SMART_READ_ATTRIBUTE_THRESHOLDS         = $D1; // Obsoleted in ATA4!
  SMART_ENABLE_DISABLE_ATTRIBUTE_AUTOSAVE = $D2;
  SMART_SAVE_ATTRIBUTE_VALUES             = $D3;
  SMART_EXECUTE_OFFLINE_IMMEDIATE         = $D4; // ATA4
  //   Vendor specific commands:
  SMART_ENABLE_SMART_OPERATIONS           = $D8;
  SMART_DISABLE_SMART_OPERATIONS          = $D9;
  SMART_RETURN_SMART_STATUS               = $DA;

type
  PDriveAttribute = ^TDriveAttribute;
  TDriveAttribute = packed record
    bAttrID: Byte;
    wStatusFlags: Word;
    bAttrValue: Byte;
    bWorstValue: Byte;
    bRawValue: array[0..5] of Byte;
    bReserved: Byte;
  end;

  // The following structure defines the structure of a Warranty Threshold
  // Obsoleted in ATA4!
  PAttrThreshold = ^TAttrThreshold;
  TAttrThreshold = packed record
    bAttrID: Byte;
    bWarrantyThreshold: Byte;
    bReserved: array[0..9] of Byte;
  end;

  // The following struct defines the interesting part of the IDENTIFY
  // buffer:
  PIdSector = ^TIdSector;
  TIdSector = record
    wGenConfig: Word;
    wNumCyls: Word;
    wReserved: Word;
    wNumHeads: Word;
    wBytesPerTrack: Word;
    wBytesPerSector: Word;
    wSectorsPerTrack: Word;
    wVendorUnique: array[0..2] of Word;
    sSerialNumber: array[0..19] of Char;
    wBufferType: Word;
    wBufferSize: Word;
    wECCSize: Word;
    sFirmwareRev: array[0..7] of Char;
    sModelNumber: array[0..39] of Char;
    wMoreVendorUnique: Word;
    wDoubleWordIO: Word;
    wCapabilities: Word;
    wReserved1: Word;
    wPIOTiming: Word;
    wDMATiming: Word;
    wBS: Word;
    wNumCurrentCyls: Word;
    wNumCurrentHeads: Word;
    wNumCurrentSectorsPerTrack: Word;
    ulCurrentSectorCapacity: ULONG;
    wMultSectorStuff: Word;
    ulTotalAddressableSectors: ULONG;
    wSingleWordDMA: Word;
    wMultiWordDMA: Word;
    bReserved: array[0..127] of Byte;
  end;

const
  // Valid Attribute IDs
  ATTR_INVALID                      = 0;
  ATTR_READ_ERROR_RATE              = 1;
  ATTR_THROUGHPUT_PERF              = 2;
  ATTR_SPIN_UP_TIME                 = 3;
  ATTR_START_STOP_COUNT             = 4;
  ATTR_REALLOC_SECTOR_COUNT         = 5;
  ATTR_READ_CHANNEL_MARGIN          = 6;
  ATTR_SEEK_ERROR_RATE              = 7;
  ATTR_SEEK_TIME_PERF               = 8;
  ATTR_POWER_ON_HRS_COUNT           = 9;
  ATTR_SPIN_RETRY_COUNT             = 10;
  ATTR_CALIBRATION_RETRY_COUNT      = 11;
  ATTR_POWER_CYCLE_COUNT            = 12;
  ATTR_SOFT_READ_ERROR_RATE         = 13;
  ATTR_G_SENSE_ERROR_RATE           = 191;
  ATTR_POWER_OFF_RETRACT_CYCLE      = 192;
  ATTR_LOAD_UNLOAD_CYCLE_COUNT      = 193;
  ATTR_TEMPERATURE                  = 194;
  ATTR_REALLOCATION_EVENTS_COUNT    = 196;
  ATTR_CURRENT_PENDING_SECTOR_COUNT = 197;
  ATTR_UNCORRECTABLE_SECTOR_COUNT   = 198;
  ATTR_ULTRADMA_CRC_ERROR_RATE      = 199;
  ATTR_WRITE_ERROR_RATE             = 200;
  ATTR_DISK_SHIFT                   = 220;
  ATTR_G_SENSE_ERROR_RATEII         = 221;
  ATTR_LOADED_HOURS                 = 222;
  ATTR_LOAD_UNLOAD_RETRY_COUNT      = 223;
  ATTR_LOAD_FRICTION                = 224;
  ATTR_LOAD_UNLOAD_CYCLE_COUNTII    = 225;
  ATTR_LOAD_IN_TIME                 = 226;
  ATTR_TORQUE_AMPLIFICATION_COUNT   = 227;
  ATTR_POWER_OFF_RETRACT_COUNT      = 228;
  ATTR_GMR_HEAD_AMPLITUDE           = 230;
  ATTR_TEMPERATUREII                = 231;
  ATTR_READ_ERROR_RETRY_RATE        = 250;

  // Status Flags Values
  PRE_FAILURE_WARRANTY      = $1;
  ON_LINE_COLLECTION        = $2;
  PERFORMANCE_ATTRIBUTE	    = $4;
  ERROR_RATE_ATTRIBUTE      = $8;
  EVENT_COUNT_ATTRIBUTE     = $10;
  SELF_PRESERVING_ATTRIBUTE = $20;

  NUM_ATTRIBUTE_STRUCTS = 30;

implementation

end.

